{
  "type": "record",
  "Aspect": {
    "name": "incidentSource"
  },
  "name": "IncidentSource",
  "namespace": "com.linkedin.pegasus2avro.incident",
  "fields": [
    {
      "Searchable": {
        "addToFilters": true,
        "filterNameOverride": "Source"
      },
      "type": {
        "type": "enum",
        "symbolDocs": {
          "ASSERTION_FAILURE": "An assertion has failed, triggering the incident.",
          "MANUAL": "Manually created incident, via UI or API."
        },
        "name": "IncidentSourceType",
        "namespace": "com.linkedin.pegasus2avro.incident",
        "symbols": [
          "MANUAL",
          "ASSERTION_FAILURE"
        ]
      },
      "name": "type",
      "doc": "Message associated with the incident"
    },
    {
      "Searchable": {
        "fieldType": "URN"
      },
      "java": {
        "class": "com.linkedin.pegasus2avro.common.urn.Urn"
      },
      "type": [
        "null",
        "string"
      ],
      "name": "sourceUrn",
      "default": null,
      "doc": "Reference to an URN related to the source of an incident. Currently populated when the type of the incident\nis ASSERTION_FAILURE.\n\nIf this incident was raised due to an assertion failure, then this will contain the urn of the source Assertion.",
      "Urn": "Urn"
    }
  ],
  "doc": "Information about the source of an incident raised on an asset."
}