{
  "type": "record",
  "Aspect": {
    "name": "share"
  },
  "name": "Share",
  "namespace": "com.linkedin.pegasus2avro.common",
  "fields": [
    {
      "Searchable": {
        "/*/destination": {
          "fieldName": "shareDestinations",
          "fieldType": "URN",
          "hasValuesFieldName": "hasShareDestinations"
        }
      },
      "type": {
        "type": "array",
        "items": {
          "type": "record",
          "name": "ShareResult",
          "namespace": "com.linkedin.pegasus2avro.common",
          "fields": [
            {
              "java": {
                "class": "com.linkedin.pegasus2avro.common.urn.Urn"
              },
              "type": "string",
              "name": "destination",
              "doc": "Urn of the DataHubConnection related to another Acryl instance",
              "Urn": "Urn"
            },
            {
              "java": {
                "class": "com.linkedin.pegasus2avro.common.urn.Urn"
              },
              "type": [
                "null",
                "string"
              ],
              "name": "implicitShareEntity",
              "default": null,
              "doc": "If this share happened implicitly due to another entity being shared, this field\nwill be populated with that entity urn. Example: if a dataset is shared, its containers\nwill also be shared. The share result for the container will have the dataset urn here.",
              "Urn": "Urn"
            },
            {
              "type": {
                "type": "record",
                "name": "AuditStamp",
                "namespace": "com.linkedin.pegasus2avro.common",
                "fields": [
                  {
                    "type": "long",
                    "name": "time",
                    "doc": "When did the resource/association/sub-resource move into the specific lifecycle stage represented by this AuditEvent."
                  },
                  {
                    "java": {
                      "class": "com.linkedin.pegasus2avro.common.urn.Urn"
                    },
                    "type": "string",
                    "name": "actor",
                    "doc": "The entity (e.g. a member URN) which will be credited for moving the resource/association/sub-resource into the specific lifecycle stage. It is also the one used to authorize the change.",
                    "Urn": "Urn"
                  },
                  {
                    "java": {
                      "class": "com.linkedin.pegasus2avro.common.urn.Urn"
                    },
                    "type": [
                      "null",
                      "string"
                    ],
                    "name": "impersonator",
                    "default": null,
                    "doc": "The entity (e.g. a service URN) which performs the change on behalf of the Actor and must be authorized to act as the Actor.",
                    "Urn": "Urn"
                  },
                  {
                    "type": [
                      "null",
                      "string"
                    ],
                    "name": "message",
                    "default": null,
                    "doc": "Additional context around how DataHub was informed of the particular change. For example: was the change created by an automated process, or manually."
                  }
                ],
                "doc": "Data captured on a resource/association/sub-resource level giving insight into when that resource/association/sub-resource moved into a particular lifecycle stage, and who acted to move it into that specific lifecycle stage."
              },
              "name": "created",
              "doc": "Audit stamp for when the first share request for the given destination occurred."
            },
            {
              "type": "com.linkedin.pegasus2avro.common.AuditStamp",
              "name": "lastAttempt",
              "doc": "When did we get this share result and who requested the share. This will be the time\nthat we try and execute the share on the backend, regardless of success."
            },
            {
              "type": [
                "null",
                "com.linkedin.pegasus2avro.common.AuditStamp"
              ],
              "name": "lastSuccess",
              "default": null,
              "doc": "Audit stamp for the last time a share to this above destination successfully occurred"
            },
            {
              "type": {
                "type": "enum",
                "symbolDocs": {
                  "FAILURE": "The share was a failure",
                  "PARTIAL_SUCCESS": "The share was partially successful. Some, but not all, aspects were shared.",
                  "SUCCESS": "The share was successful"
                },
                "name": "ShareResultState",
                "namespace": "com.linkedin.pegasus2avro.common",
                "symbols": [
                  "SUCCESS",
                  "PARTIAL_SUCCESS",
                  "FAILURE"
                ]
              },
              "name": "status",
              "doc": "Status for this share result - whether or not it succeeded"
            },
            {
              "type": [
                "null",
                "string"
              ],
              "name": "message",
              "default": null,
              "doc": "Optional message for this share result. Error messages will be stored here."
            }
          ],
          "doc": "Info about the results from sharing metadata for a given entity"
        }
      },
      "name": "lastShareResults",
      "doc": "The last share result for each destination that this entity has been shared with"
    }
  ],
  "doc": "Aspect used for keeping track of metadata sharing info"
}